@@ -2690,7 +2690,7 @@ int main(int argc, char *argv[])
 {
 	char *source;
 	char *target;
-	u64 num_threads = 1;
+	u64 num_threads = 0;
 	u64 compress_level = 0;
 	int create = 1;
 	int old_restore = 0;
@@ -2786,10 +2786,14 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (num_threads == 1 && compress_level > 0) {
-		num_threads = sysconf(_SC_NPROCESSORS_ONLN);
-		if (num_threads <= 0)
-			num_threads = 1;
+	if (compress_level > 0 || create == 0) {
+		if (num_threads == 0) {
+			num_threads = sysconf(_SC_NPROCESSORS_ONLN);
+			if (num_threads <= 0)
+				num_threads = 1;
+		}
+	} else {
+		num_threads = 0;
 	}
 
 	if (create) {
@@ -2877,5 +2881,7 @@ out:
 		}
 	}
 
+	btrfs_close_all_devices();
+
 	return !!ret;
 }
