@@ -35,7 +35,7 @@ generate_leaf_corrupt_no_data_ext()
 {
 	dest=$1
 	echo "generating leaf_corrupt_no_data_ext.btrfs-image" >> $RESULTS
-	tar xJf ./no_data_extent.tar.xz || \
+	tar --no-same-owner -xJf ./no_data_extent.tar.xz || \
 		_fail "failed to extract leaf_corrupt_no_data_ext.btrfs-image"
 	$TOP/btrfs-image -r test.img.btrfs-image $dest || \
 		_fail "failed to extract leaf_corrupt_no_data_ext.btrfs-image"
@@ -85,8 +85,7 @@ check_inode()
 check_leaf_corrupt_no_data_ext()
 {
 	image=$1
-	mkdir -p $TEST_MNT || _fail "failed to create mount point"
-	$SUDO_HELPER mount $image -o ro $TEST_MNT
+	$SUDO_HELPER mount -o loop $image -o ro $TEST_MNT
 
 	i=0
 	while [ $i -lt ${#leaf_no_data_ext_list[@]} ]; do
